/* Programmname                                                             */

/*--------------------------------------------------------------------------*/
/* System Includes                                                          */
/*--------------------------------------------------------------------------*/

	#include <clib/exec_protos.h>
	#include <clib/alib_protos.h>
	#include <clib/intuition_protos.h>
	#include <clib/graphics_protos.h>
	#include <clib/dos_protos.h>
	#include <proto/cybergraphics.h>
	#include <exec/memory.h>
	#include <clib/datatypes_protos.h>
	#include <datatypes/pictureclass.h>
	#include <cybergraphx/cybergraphics.h>

	#include <stdlib.h>
	#include <string.h>
	#include <stdio.h>

/*--------------------------------------------------------------------------*/
/* Konstanten und Makros                                                    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Typdefinitionen                                                          */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Globale Variablen                                                        */
/*--------------------------------------------------------------------------*/

	struct Library *CyberGfxBase = NULL;

/*--------------------------------------------------------------------------*/
/* Prototypen                                                               */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* create mask from bitmap by extracting one color                          */
/*--------------------------------------------------------------------------*/

	struct BitMap *create_mask (struct BitMap *srcbm,long w,long h,long bgx,long bgy)

/*

parameters:

  srcbm   - bitmap to create mask from
  w,h     - size of picture
  x,y     - position in the picture where to fetch transparent color from (usually 0,0)

result:

  maskbm  - bitmap containing mask

*/

	{
	struct BitMap *maskbm;
	long d,x,y;
	long bpr;
	struct RastPort rp;
	struct TmpRas tmpras;
	UBYTE *tmpbuf;
	ULONG rassize;
	struct RastPort temprp;
	struct BitMap *tempbm;
	UBYTE *array,*yp,*xp,*p;
	long tpen;
	ULONG trgb;
	ULONG rgb;

	if (maskbm = AllocBitMap (w,h,2,BMF_CLEAR,NULL)) /* depth 2 because we create two masks (see backfill code below) */
		{
		InitRastPort (&rp);
		rp.BitMap = srcbm;

		if (tempbm = AllocBitMap (w,h,1,0,NULL))
			{
			temprp = rp;                 /* for Read/WritePixelArray8 as documented in autodocs */
			temprp.Layer = NULL;
			temprp.BitMap = tempbm;

			bpr = ((w + 15) >> 4) << 4;    /* bytes per row for Read/WritePixelArray8 */

			d = GetBitMapAttr(srcbm,BMA_DEPTH);
			if (d <= 8)
				{
				if (array = AllocVec (bpr * h,0))
					{
					ReadPixelArray8 (&rp,0,0,w-1,h-1,array,&temprp);

					tpen = array[bgy * bpr + bgx];   /* fetch transparent pen */

					yp = array;
					for (y = 0; y < h; y++)
						{
						xp = yp;
						for (x = 0; x < w; x++)
							{
							*xp++ = (*xp == tpen ? 0 : 1); /* set transparent pixels to 0, others to 1 */
							}
						yp += bpr;
						}
					}
				}
			else if (CyberGfxBase)
				{
				if (array = AllocVec (w * h * 3,0))
					{
					ReadPixelArray (array,0,0,w*3,&rp,0,0,w,h,RECTFMT_RGB);

					p = &array[(bgy * w + bgx) * 3];            /* pointer to background color */
					trgb = (*p++ << 16) | (*p++ << 8) | (*p++); /* get transparent RGB value */

					p = array;
					yp = array;

				/* note that array is used in two different ways at the same
				   time here: three bytes per pixel (RGB) as taken from
				   ReadPixelArray are read and one byte per pixel (pen number)
				   is written to prepare WritePixelArray8. This can be done
				   because the read pointer is faster than the write pointer,
				   so the writes do not destroy data not yet read.
   				*/
					for (y = 0; y < h; y++)
						{
						xp = yp;
						for (x = 0; x < w; x++)
							{
							rgb = (*p++ << 16) | (*p++ << 8) | (*p++);
							*xp++ = (rgb == trgb ? 0 : 1);
							}
						yp += bpr;
						}
					}
				}

			if (array)
				{
				rp.BitMap = maskbm;

				WritePixelArray8 (&rp,0,0,w-1,h-1,array,&temprp); /* write 0/1 pixels into maskbm => bitplane 0 contains mask */

				FreeVec (array);

		/* the following is only needed for backfill */

				rassize = RASSIZE(w,h);
				if (tmpbuf = AllocVec (rassize,MEMF_CHIP|MEMF_CLEAR|MEMF_PUBLIC))
					{
					InitTmpRas (&tmpras,tmpbuf,rassize);
					rp.TmpRas = &tmpras;
					}

				SetAPen (&rp,2);
				Flood (&rp,1,bgx,bgy);  /* fill area around mask with pen 2 => bitplane 1 contains filled background */

				BltBitMap (maskbm,0,0,maskbm,0,0,w,h,ABNC|ANBNC,0x02,NULL);   /* invert bitplane 1 => bp 1 contains mask without holes in it */

				FreeVec (tmpbuf);

		/* end of backfill code */

				}

			FreeBitMap (tempbm);
			}
		}

	return (maskbm);
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void free_mask (struct BitMap *bm)

	{
	FreeBitMap (bm);
	}

/*--------------------------------------------------------------------------*/
/* Hauptprogramm                                                            */
/*--------------------------------------------------------------------------*/

	int main (int argc,char *argv[])

	{
	struct RDArgs *rdargs;
	static long zero = 0;
	static struct {
		char *from;
		long *x;
		long *y;
		long backfill;
		long v43;
		char *pubscreen;
		} args = {"leer:eigene dateien/pix/misc/petereggs.gif",&zero,&zero,0,0,NULL};

	struct Screen *scr;
	Object *o;
	struct BitMapHeader *bmhd;
	struct BitMap *srcbm,*destbm,*maskbm;
	long w,h,x,y;
	struct Window *win;
	struct RastPort *rp;
	struct MsgPort *port;
	struct IntuiMessage *mess;
	BOOL cont;
	char buffer[80];

	CyberGfxBase = OpenLibrary ("cybergraphics.library",0); /* only needed for >8 bit screens */

	if (rdargs = ReadArgs ("FROM,X/N,Y/N,BACKFILL/S,V43/S,PUBSCREEN/K",(APTR)&args,NULL))
		{

		if (scr = LockPubScreen (args.pubscreen))
			{
			if (o = NewDTObject(args.from,
					DTA_GroupID,GID_PICTURE,
					PDTA_Remap,TRUE,
					PDTA_Screen,scr,
					PDTA_FreeSourceBitMap,FALSE,
					TAG_END))
				{
				if (args.v43)			/* use V43 mode if the user wishes to */
					SetDTAttrs (o,NULL,NULL,PDTA_DestMode,PMODE_V43,TAG_END);

				DoMethod (o,DTM_PROCLAYOUT,NULL,1);	/* create bitmaps */

				GetDTAttrs (o,
					PDTA_BitMapHeader,&bmhd,
					PDTA_BitMap,&srcbm,
					PDTA_DestBitMap,&destbm,
					TAG_END);

				w = bmhd->bmh_Width;
				h = bmhd->bmh_Height;

				if (w > 300) w = 300;	/* let's fit the window onto the screen */
				if (h > 300) h = 300;

				if (win = OpenWindowTags (NULL,
						WA_InnerWidth,3*w,
						WA_InnerHeight,h,
						WA_Title,"Mask",
						WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_ACTIVATE|WFLG_GIMMEZEROZERO|WFLG_REPORTMOUSE,
						WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_MOUSEBUTTONS|IDCMP_MOUSEMOVE|IDCMP_VANILLAKEY,
						TAG_END))
					{
					rp = win->RPort;

					BltBitMapRastPort (destbm,0,0,rp,0,0,w,h,ABC|ABNC);

					if (maskbm = create_mask (srcbm,bmhd->bmh_Width,bmhd->bmh_Height,*args.x,*args.y))
						{
						BltBitMapRastPort (maskbm,0,0,rp,w,0,w,h,ABC|ABNC);
						BltMaskBitMapRastPort (destbm,0,0,rp,2*w,0,w,h,ABC|ABNC|ANBC,maskbm->Planes[args.backfill ? 1 : 0]);

						free_mask (maskbm);
						}

/*--------------------------------------------------------------------------------------------

MinTerms as documented in autodocs:

             A B C  vanilla copy  invert copy  invert dest  blit thru mask  invert & mask
NANBNC  01   0 0 0      0             0            0              0               0
NANBC   02   0 0 1      0             0            0              0               0
NABNC   04   0 1 0      0             0            0              0               0
NABC    08   0 1 1      0             0            0              0               0
ANBNC   10   1 0 0      0             1            1              0               0
ANBC    20   1 0 1      0             1            0              1               1
ABNC    40   1 1 0      1             0            1              1               0
ABC     80   1 1 1      1             0            0              1               0

----------------------------------------------------------------------------------------------*/


					port = win->UserPort;
					cont = TRUE;
					while (cont)
						{
						WaitPort (port);
						while (mess = (struct IntuiMessage *) GetMsg (port))
							{
							switch (mess->Class)
								{
							case IDCMP_MOUSEMOVE:
								x = win->GZZMouseX;
								y = win->GZZMouseY;
								if (x >= 0 && x < 3*w && y >= 0 && y < h)
									{
									sprintf (buffer,"Mask          x: %ld   y: %ld",x % w,y);
									SetWindowTitles (win,buffer,(UBYTE *)-1);
									}
								else
									SetWindowTitles (win,"Mask",(UBYTE *)-1);
								break;
							default:
								cont = FALSE;
								}
							ReplyMsg ((struct Message *)mess);
							}
						}
					CloseWindow (win);
					}

				DisposeDTObject (o);
				}
			else
				PrintFault (IoErr(),NULL);

			UnlockPubScreen (NULL,scr);
			}

		FreeArgs (rdargs);
		}
	else
		PrintFault (IoErr(),NULL);

	if (CyberGfxBase) CloseLibrary (CyberGfxBase);

	return(0);
	}

/*--------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                     */
/*--------------------------------------------------------------------------*/
